#include "dvbs.h"

static struct {
        struct time t;
        struct date d;
        struct timehms hms;
        char ct[9];
} tm;

static void checktime(void)
{
        struct time t;
        unsigned int x;

        t = gettime();
        if (t.day != tm.t.day)
                tm.d = mjdtodate(t.day);
        if (t.sec != tm.t.sec)
                timedec(tm.ct, &tm.hms, t.sec);
        tm.t = t;
}

static const char pg_header[] = "\xc0 www.D-ATV.de        \xc1";

static const char * const pg_100[] = {
        pg_header,
        NULL,
        "\x01 panelkovej MUX",
        NULL,
        "Digital Baseband:",
        "  Martin Patera, OK1MJO",
        NULL,
        "RF",
        "  Wolf-Henning Rech, DF9IC/N1EOW",
        "  Jens Geisler, DL8SDL",
        NULL,
        "Schematics, Boards &",
        " Connections to Fujitsu",
        "  Stefan Reimann, DG8FAC",
        NULL,
        "\x03""adacom e.V.",
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        "\x01\x1d\x80Label 1\x1c\x02\x1d\x80Label 2\x1c\x03\x1d\x80Label 3\x1c\x04\x1d\x80Label 4\x1c",
        "Label 1   Label 2    Label 3   Label 4"
};

static const char * const pg_101[] = {
        pg_header,
        NULL,
        "\x05Multiplexer Statistics Page (hex)",
        NULL,
        "Local PCR                      \xc2\x08",
        "Total packet count          \x06  \xc3\x08",
        "Mux-generated NULL packets     \xc4\x08",
        "Table/Teletext packets         \xc5\x08",
        "Transport Stream 1 packets     \xc6\x08",
        "Transport Stream 2 packets     \xc7\x08",
        "Transport Stream 3 packets     \xc8\x08",
        "Transport Stream 4 packets     \xc9\x08",
        "Unused                         \xca\x08",
        "Unused                         \xcb\x08",
        "Unused                         \xcc\x08",
        "Unused                         \xcd\x08",
        "Unused                         \xce\x08",
        "Unused                         \xcf\x08",
        "Unused                         \xd0\x08",
        "Unused                         \xd1\x08",
        NULL,
        NULL,
        NULL,
        NULL,
        NULL
};

static const char * const pg_102[] = {
        pg_header,
        NULL,
        "\x05Multiplexer Statistics Page (dec)",
        NULL,
        "Local PCR                    \xc2\x08",
        "Total packet count          \x06\xc3\x08",
        "Mux-generated NULL packets   \xc4\x08",
        "Table/Teletext packets       \xc5\x08",
        "Transport Stream 1 packets   \xc6\x08",
        "Transport Stream 2 packets   \xc7\x08",
        "Transport Stream 3 packets   \xc8\x08",
        "Transport Stream 4 packets   \xc9\x08",
        "Unused                       \xca\x08",
        "Unused                       \xcb\x08",
        "Unused                       \xcc\x08",
        "Unused                       \xcd\x08",
        "Unused                       \xce\x08",
        "Unused                       \xcf\x08",
        "Unused                       \xd0\x08",
        "Unused                       \xd1\x08",
        NULL,
        NULL,
        NULL,
        NULL,
        NULL
};

static const char *pg_103[] = {
        pg_header,
        NULL,
        "\x05Status Page",
        NULL,
        "Time    \xc1.\xc2",
        "Date      \xc3, \xc5. \xc4 \xc7",
        "          \xc3, \xc5.\xc6.\xc7",
        "MJD            \xc8",
        "Jiffies        \xc9",
        "Teletext Cycle \xca",
        NULL,
        "Analog 0: \xcb",
        "Analog 1: \xcc",
        "Analog 2: \xcd",
        "Analog 3: \xce",
        NULL,
        "TS1 ServiceName         \xcf",
        "TS1 ServiceProviderName \xd0",
        NULL,
        "TS2 ServiceName         \xd1",
        "TS2 ServiceProviderName \xd2",
        NULL,
        NULL,
        NULL,
        NULL
};

static const char *pg_104[] = {
        pg_header,
        NULL,
        "\x05Tuner \xc2 Status Page",
        NULL,
        "\x04Parameters",
        NULL,
        "Frequency           \xc3 kHz",
        "Symbol Rate         \xc4 kSymbols/s",
        "FEC                 \xc5",
        "Spectral Inversion  \xc6",
        NULL,
        "\x04Status",
        NULL,
        "Signal              \xc7 \xc8 \xc9 \xca",
        "Frequency Offset    \xcb kHz",
        "FEC                 \xcc",
        "BER                 \xcd",
        "AGC (Signal Level)  \xce / \xcf",
        "CNR                 \xd0",
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL
};

static const char * const pg_111[] = {
        pg_header,
        "01",
        "02",
        "03",
        "04",
        "05",
        "06",
        "07",
        "08",
        "09",
        "10",
        "11",
        "12",
        "13",
        "14",
        "15",
        "16",
        "17",
        "18",
        "19",
        "20",
        "21",
        "22",
        "23",
        "24"
};

static const char * const pg_177[] = {
        pg_header,
        NULL,
        "\x05Last Page",
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL
};

static const char * const pg_20x[] = {
        pg_header,
        NULL,
        TXTATTR_ALPHA_MAGENTA "Event Log",
        NULL,
        "\xc2",
        "  \xc3",
        "  \xc4",
        "  \xc5",
        "\xc6",
        "  \xc7",
        "  \xc8",
        "  \xc9",
        "\xca",
        "  \xcb",
        "  \xcc",
        "  \xcd",
        "\xce",
        "  \xcf",
        "  \xd0",
        "  \xd1",
        "\xd2",
        "  \xd3",
        "  \xd4",
        "  \xd5",
        NULL
};

static void logpages(void)
{
        static unsigned int pg = ~0, ptr;
        unsigned int i, j;
        char num[4];
        char buf[5][154];

        if (pg > 0x09) {
                logreadinit(&ptr);
                pg = 0;
        }
        for (i = 0; i < 5; i++) {
                j = logreadline(&ptr, buf[i], sizeof(buf[i]));
                if (j < sizeof(buf[i]))
                        memset(&buf[i][j], 0, sizeof(buf[i])-j);
                if (j && buf[i][j-1] == '\n')
                        buf[i][j-1] = 0;
        }
        checktime();
        i = 0x200 | pg;
        teletext_encodepage(0, 24, i, 0, TXTPAGECTRL_UPDATEINDICATOR, pg_20x, int2hex(num, 3, i, INTCONV_PADZERO), tm.ct,
                            buf[0], buf[0]+40, buf[0]+78, buf[0]+116,
                            buf[1], buf[1]+40, buf[1]+78, buf[1]+116,
                            buf[2], buf[2]+40, buf[2]+78, buf[2]+116,
                            buf[3], buf[3]+40, buf[3]+78, buf[3]+116,
                            buf[4], buf[4]+40, buf[4]+78, buf[4]+116);
        teletext_encodepage(0, 0, 0x2ff, 0, 0, pg_20x, num, tm.ct);
        pg++;
}

static void tunerpages(void)
{
        static const u_int8_t invmode0[] = "off";
        static const u_int8_t invmode1[] = "on";
        static const u_int8_t invmode2[] = "auto";
        static const u_int8_t *invmode[] = { invmode0, invmode1, invmode2 };
        static const u_int8_t fecmode[24] = "1/2\000""2/3\000""3/4\000""5/6\000""7/8\000""---\000";
        static const u_int8_t syncoff[] = "   ";
        static const u_int8_t syncon[] = "DVB";
        static const u_int8_t fecoff[] = "   ";
        static const u_int8_t fecon[] = "FEC";
        static const u_int8_t caroff[] = "   ";
        static const u_int8_t caron[] = "CAR";
        static const u_int8_t looff[] = "  ";
        static const u_int8_t loon[] = "LO";
        struct tuner_capability tcap;
        struct tuner_parameter tpar;
        struct tuner_status tstat;
        char buf[10][32];
        char *b[10];
        unsigned int i;

        for (i = 0; i < 4; i++) {
                if (tuner_getcapability(i, &tcap))
                        continue;
                if (tcap.tunertype != TUNERTYPE_DVBS)
                        continue;
                if (tuner_getparameters(i, &tpar))
                        continue;
                if (tuner_getstatus(i, &tstat))
                        continue;
                strcpy(buf[0], "104");
                buf[0][2] += i;
                buf[1][0] = '1' + i;
                buf[1][1] = 0;
                b[2] = int2dec(buf[2], 7, tpar.freq, INTCONV_PADSPACE);
                b[3] = int2dec(buf[3], 5, tpar.symrate, INTCONV_PADSPACE);
                buf[4][0] = buf[4][1] = 0;
                if (tpar.fecmask & 1)
                        strcat(buf[4], " 1/2");
                if (tpar.fecmask & 2)
                        strcat(buf[4], " 2/3");
                if (tpar.fecmask & 4)
                        strcat(buf[4], " 3/4");
                if (tpar.fecmask & 8)
                        strcat(buf[4], " 5/6");
                if (tpar.fecmask & 16)
                        strcat(buf[4], " 7/8");
                b[5] = int2dec(buf[5], 6, tstat.freqoffs, INTCONV_SIGN|INTCONV_PLUS|INTCONV_PADSPACE);
                b[6] = int2dec(buf[6], 9, tstat.ber, INTCONV_PADSPACE);
                b[7] = int2dec(buf[7], 5, tstat.agc1, INTCONV_PADSPACE);
                b[8] = int2dec(buf[8], 5, tstat.agc2, INTCONV_PADSPACE);
                b[9] = int2dec(buf[9], 5, tstat.cnr, INTCONV_PADSPACE);
                checktime();
                teletext_encodepage(0, 24, 0x104+i, 0, TXTPAGECTRL_UPDATEINDICATOR, pg_104, buf[0], tm.ct,
                                    buf[1], b[2], b[3], buf[4]+1, invmode[tpar.invmode],
                                    (tstat.sync & TUNERSYNC_LOLOCK) ? loon : looff,
                                    (tstat.sync & TUNERSYNC_CARRIER) ? caron : caroff,
                                    (tstat.sync & TUNERSYNC_FECPUNCT) ? fecon : fecoff,
                                    (tstat.sync & TUNERSYNC_SYNC) ? syncon : syncoff,
                                    b[5], fecmode + 4 * tstat.fecmode, b[6], b[7], b[8], b[9]);
                teletext_encodepage(0, 0, 0x1ff, 0, 0, pg_104, buf[0], tm.ct);
        }
}

void teletext(void)
{
        static const u_int8_t weekdays[28] = "Wed\000Thu\000Fri\000Sat\000Son\000Mon\000Tue\000";
        static const u_int8_t months[52] = "???\000Jan\000Feb\000Mar\000Apr\000May\000Jun\000Jul\000Aug\000Sep\000Oct\000Nov\000Dec\000";
        u_int8_t *lp;
        unsigned int txtcycle = 0;
        char buf[16][16];
        char *b[16];
        unsigned int i;
        struct portcapture cap0, cap1;

        for (;;) {
                checktime();
                teletext_encodepage(0, 24, 0x100, 0, 0, pg_100, "100", tm.ct);
                teletext_encodepage(0, 0, 0x1ff, 0, 0, pg_100, "100", tm.ct);

                checktime();
                for (i = 0; i < 16; i++)
                        b[i] = int2hex(buf[i], 8, readcounter(i), INTCONV_PADZERO);
                teletext_encodepage(0, 24, 0x101, 0, TXTPAGECTRL_UPDATEINDICATOR, pg_101, "101", tm.ct,
                                    buf[0], buf[1], buf[2], buf[3], buf[4], buf[5], buf[6], buf[7],
                                    buf[8], buf[9], buf[10], buf[11], buf[12], buf[13], buf[14], buf[15]);
                teletext_encodepage(0, 0, 0x1ff, 0, 0, pg_101, "101", tm.ct);

                checktime();
                for (i = 0; i < 16; i++)
                        b[i] = int2dec(buf[i], 10, readcounter(i), INTCONV_PADSPACE);
                teletext_encodepage(0, 24, 0x102, 0, TXTPAGECTRL_UPDATEINDICATOR, pg_102, "102", tm.ct,
                                    b[0], b[1], b[2], b[3], b[4], b[5], b[6], b[7],
                                    b[8], b[9], b[10], b[11], b[12], b[13], b[14], b[15]);
                teletext_encodepage(0, 0, 0x1ff, 0, 0, pg_102, "102", tm.ct);

                checktime();
                b[0] = int2dec(buf[0], 3, tm.t.msec, INTCONV_PADZERO);
                b[1] = int2dec(buf[1], 2, tm.d.d, 0);
                b[2] = int2dec(buf[2], 2, tm.d.m, 0);
                b[3] = int2dec(buf[3], 4, tm.d.y, 0);
                b[4] = int2dec(buf[4], 10, tm.t.day, 0& INTCONV_PADSPACE);
                b[5] = int2dec(buf[5], 10, getjiffies(), 0& INTCONV_PADSPACE);
                b[6] = int2dec(buf[6], 10, txtcycle, 0& INTCONV_PADSPACE);
                b[7] = int2dec(buf[7], 4, getadc(0), 0& INTCONV_PADSPACE);
                b[8] = int2dec(buf[8], 4, getadc(1), 0& INTCONV_PADSPACE);
                b[9] = int2dec(buf[9], 4, getadc(2), 0& INTCONV_PADSPACE);
                b[10] = int2dec(buf[10], 4, getadc(3), 0& INTCONV_PADSPACE);
                cap0 = getcapture(0);
                cap1 = getcapture(1);
                teletext_encodepage(0, 24, 0x103, 0, TXTPAGECTRL_UPDATEINDICATOR, pg_103, "103", tm.ct,
                                    b[0], weekdays + 4 * (tm.t.day % 7), months + 4 * tm.d.m, b[1], b[2], b[3],
                                    b[4], b[5], b[6], b[7], b[8], b[9], b[10],
                                    cap0.service_name, cap0.service_provider_name,
                                    cap1.service_name, cap1.service_provider_name);
                teletext_encodepage(0, 0, 0x1ff, 0, 0, pg_103, "103", tm.ct);

                tunerpages();

                checktime();
                teletext_encodepage(0, 24, 0x111, 0, 0, pg_111, "111", tm.ct);
                teletext_encodepage(0, 0, 0x1ff, 0, 0, pg_111, "111", tm.ct);

                checktime();
                teletext_encodepage(0, 24, 0x177, 0, 0, pg_177, "177", tm.ct);
                teletext_encodepage(0, 0, 0x1ff, 0, 0, pg_177, "177", tm.ct);

                logpages();

                txtcycle++;
        }
}